
/*************** Practica 02. Implementacion clase Conjunto********************
 *
 *  Nombre de Fichero: conjunto.cpp
 *
 *  Proposito: En este archivo se implementan los diferentes metodos de la
 *             clase conjunto
 *
 *  Condiciones de Final Anormal y Mensajes de Error y Advertencia
 *
 *  --> No hay final anormal
 *  --> No hay mensajes de error y advertencia
 *
 *  Historia de Desarrollo:
 *  Fecha     Autor           Id Cambio  Version  Descripcion del cambio
 *  --------  --------------  ---------  -------  ----------------------
 *  15-03-02  Ricardo Ferris          0      1.0
 *  15-03-04  Jesus Albert            1      1.1   Adaptacion a cambios en
 *                                                 enunciado
 *****************************************************************************/

#include <iostream>
using namespace std;
#include "00conjunto.h"

/*************** IniciarConjunto *****************************************
 *
 *  Prototipo:    void Conjunto::IniciarConjunto (void)
 *
 *  Descripcion:  Este metodo inicia el conjunto a vacio (pone el valor
 *                privado 'n' a cero)
 *
 *  Parametros:
 *  Nombre          Tipo        E/S  Descripcion
 *  ------          ----------  ---  ------------
 *  void
 *
 *  Valor devuelto:
 *  void
 *
 ************************************************************************/
void Conjunto::IniciarConjunto (void)
{
   n = 0;
}

/*************** AnadirElemento ******************************************
 *
 *  Prototipo:    bool Conjunto::AnadirElemento (Conjunto::Elemento x)
 *
 *  Descripcion:  Este metodo anade el elemento x al conjunto. Si el
 *                elemento existia en el conjunto, no lo guarda.
 *                Si no caben mas elementos en el conjunto devuelve error
 *
 *  Parametros:
 *  Nombre          Tipo        E/S  Descripcion
 *  ------          ----------  ---  ------------
 *  x               Elemento    E    Valor que quiero guardar en el
 *                                   conjunto
 *
 *  Valor devuelto:
 *  bool    Exito en la insercion del elemento
 *          true  -> El elemento ha sido anadido correctamente
 *          false -> No es posible aadir el elemento en el conjunto
 *
 ************************************************************************/
bool Conjunto::AnadirElemento (Conjunto::Elemento x)
{
   bool ok;
   int i;

   /*
    * Si 'n' vale Max ya no me caben mas elementos en el conjunto
    */
   if (n == MAX)
      ok = false;
   else
   {
      ok = true;
      /*
       * Miro si el elemento 'x' ya estaba en el conjunto
       */
      i = 0;
      //Utilizo un centinela para mejorar la busqueda
      v[n] = x;
      while ( v[i] != x )
         i++;

      /*
       * Si no estaba en el conjunto (no lo he encontrado en el vector),
       * lo pongo al final
       */
      if (i == n)
      {
         v[n] = x;
         n++;
      }
   }

   return ok;
}

/*************** BorrarElemento ******************************************
 *
 *  Prototipo:    bool Conjunto::BorrarElemento (Conjunto::Elemento x)
 *
 *  Descripcion:  Este metodo elimina el elemento x del conjunto. Si el
 *                elemento existia en el conjunto, lo elimina.
 *                Sino no hace nada especial.
 *
 *  Parametros:
 *  Nombre          Tipo        E/S  Descripcion
 *  ------          ----------  ---  ------------
 *  x               Elemento    E    Valor que quiero eliminar del
 *                                   conjunto
 *  Valor devuelto:
 *  void
 *
 ************************************************************************/
void Conjunto::BorrarElemento (Conjunto::Elemento x)
{
   int i, j;

   /*
    * Busco el elemento en el conjunto
    */
   i = 0;
   //Utilizo un centinela para mejorar la busqueda
   v[n] = x;
   while ( v[i] != x )
      i++;

  /*
   * Si encuentro el elemento, lo elimino del vector,
   * colocando en su lugar el que ocupe la ultima posicion
   * y digo que tengo un elemento menos (n--)
   */
   if (i < n)
   {
      v[i] = v[n-1];
      n--;
   }
}

/*************** EsConjuntoVacio ******************************************
 *
 *  Prototipo:    bool Conjunto::EsConjuntoVacio (void)
 *
 *  Descripcion:  Este metodo comprueba si el conjunto esta vacio.
 *
 *  Parametros:
 *  Nombre          Tipo        E/S  Descripcion
 *  ------          ----------  ---  ------------
 *  void                                                         l
 *
 *  Valor devuelto:
 *  bool    Resultado de la comprobacion
 *          true  -> El conjunto esta vacio
 *          false -> El conjunto tienen elementos
 *
 ************************************************************************/
bool Conjunto::EsConjuntoVacio (void)
{
   return n == 0;
}

/*************** Contiene ******************************************
 *
 *  Prototipo:    bool Conjunto::Contiene (Conjunto::Elemento)
 *
 *  Descripcion:  Este metodo comprueba si un elemento pertenece al conjunto
 *
 *  Parametros:
 *  Nombre          Tipo        E/S  Descripcion
 *  ------          ----------  ---  ------------
 *  x               Elemento    E    Valor que quiero saber si pertenece al
 *                                   conjunto
 *
 *  Valor devuelto:
 *  bool    Resultado de la comprobacion
 *          true  -> El conjunto contiene al elemento
 *          false -> El elemento no esta en el conjunto
 *
 ************************************************************************/
bool Conjunto::Contiene (Conjunto::Elemento x)
{
   bool b_aux;
   int i;

   i = 0;
   //Utilizo un centinela para mejorar la busqueda
   v[n] = x;
   while ( v[i] != x )
      i++;

   if (i == n)
      b_aux = false;
   else
      b_aux = true;

   return b_aux;
}

/*************** MostrarConjunto ******************************************
 *
 *  Prototipo:    bool Conjunto::MostrarConjunto (void)
 *
 *  Descripcion:  Este metodo muestra por pantalla los datos del conjunto
 *                No existe ningun tipo de orden en el conjunto.
 *
 *  Parametros:
 *  Nombre          Tipo        E/S  Descripcion
 *  ------          ----------  ---  ------------
 *  void
 *
 *  Valor devuelto:
 *  void
 *
 ************************************************************************/
void Conjunto::MostrarConjunto ()
{
   int i;

   cout << "{ ";
   for (i = 0; i < n - 1; i++)
      cout << v[i] << ", ";

   if (n > 0)
      cout << v[n-1];
   cout << " }" << endl;

}

